@@ -34,7 +34,6 @@ Module responsible for handling the command line arguments for Gramps.
 # Standard python modules
 #
 #-------------------------------------------------------------------------
-from __future__ import print_function
 import os
 import sys
 
@@ -416,7 +415,9 @@ class ArgHandler(object):
                 for item in sorted(summary):
                     if item != "Family Tree":
                         # translators: needed for French, ignore otherwise
-                        print(_("   %s: %s") % (item, summary[item]))
+                        print(_("   %(item)s: %(summary)s") % {
+                                        'item' : item,
+                                        'summary' : summary[item] } )
             sys.exit(0)
            
         # Handle the "-t" List Family Trees, tab delimited option.  
@@ -434,10 +435,11 @@ class ArgHandler(object):
             print("\t".join(line_list))
             for summary in sorted(summary_list,
                                   key=lambda sum: sum[_("Family Tree")].lower()):
-                line_list = [('"%s"' % summary[_("Family Tree")])]
+                line_list = [(_('"%s"') % summary[_("Family Tree")])]
                 for item in sorted(summary):
                     if item != _("Family Tree"):
-                        line_list += [('"%s"' % summary[item])]
+                        # translators: ignore unless your quotation marks differ
+                        line_list += [(_('"%s"') % summary[item])]
                 print("\t".join(line_list))
             sys.exit(0)
 
