@@ -597,6 +597,8 @@ class GuiGramplet(object):
                                                        int(event.x), 
                                                        int(event.y))
         iter = view.get_iter_at_location(*buffer_location)
+        if isinstance(iter, tuple):
+            iter = iter[1]
         cursor = self.standard_cursor
         ttip = None
         for (tag, link_type, handle, tooltip) in self._tags:
@@ -619,6 +621,8 @@ class GuiGramplet(object):
                                                        int(event.x), 
                                                        int(event.y))
         iter = view.get_iter_at_location(*buffer_location)
+        if isinstance(iter, tuple):
+            iter = iter[1]
         for (tag, link_type, handle, tooltip) in self._tags:
             if iter.has_tag(tag):
                 if link_type == 'Person':
@@ -729,7 +733,7 @@ class GuiGramplet(object):
 
     def set_has_data(self, value):
         if isinstance(self.pane, Gtk.Notebook):
-            if self.pane.get_tab_label(self):
+            if self.pane.page_num(self) != -1:
                 label = self.pane.get_tab_label(self)
                 label.set_has_data(value)
 
@@ -1142,7 +1146,7 @@ class GrampletPane(Gtk.ScrolledWindow):
         if filename and os.path.exists(filename):
             cp = configparser.ConfigParser()
             try:
-                cp.read(filename)
+                cp.read(filename, encoding='utf-8')
             except Exception as err:
                 LOG.warning("Failed to load gramplets from %s because %s",
                             filename, str(err))
