@@ -19,12 +19,11 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
 "Import from CSV Spreadsheet"
 
-from __future__ import unicode_literals
 #-------------------------------------------------------------------------
 #
 # Standard Python Modules
@@ -33,6 +32,7 @@ from __future__ import unicode_literals
 import time
 import csv
 import codecs
+from io import TextIOWrapper
 
 #------------------------------------------------------------------------
 #
@@ -44,21 +44,25 @@ LOG = logging.getLogger(".ImportCSV")
 
 #-------------------------------------------------------------------------
 #
-# GRAMPS modules
+# Gramps modules
 #
 #-------------------------------------------------------------------------
 from gramps.gen.const import GRAMPS_LOCALE as glocale
 _ = glocale.translation.sgettext
 ngettext = glocale.translation.ngettext # else "nearby" comments are ignored
-from gramps.gen.lib import ChildRef, Citation, Event, EventRef, EventType, Family, FamilyRelType, Name, NameType, Note, NoteType, Person, Place, Source, Surname, Tag
+from gramps.gen.lib import (ChildRef, Citation, Event, EventRef, EventType,
+                            Family, FamilyRelType, Name, NameType, Note,
+                            NoteType, Person, Place, Source, Surname, Tag,
+                            PlaceName, PlaceType, PlaceRef)
 from gramps.gen.db import DbTxn
-from gramps.gen.plug.utils import OpenFileOrStdin
 from gramps.gen.datehandler import parser as _dp
 from gramps.gen.utils.string import gender as gender_map
 from gramps.gen.utils.id import create_id
+from gramps.gen.utils.location import located_in
 from gramps.gen.lib.eventroletype import EventRoleType
-from gramps.gen.constfunc import cuni, conv_to_unicode, STRTYPE
 from gramps.gen.config import config
+from gramps.gen.display.place import displayer as place_displayer
+from gramps.gen.utils.libformatting import ImportInfo
 
 #-------------------------------------------------------------------------
 #
@@ -76,54 +80,6 @@ def get_primary_event_ref_from_type(dbase, person, event_name):
                 return ref
     return None
 
-#-------------------------------------------------------------------------
-#
-# Encoding support for CSV, from http://docs.python.org/lib/csv-examples.html
-#
-#-------------------------------------------------------------------------
-class UTF8Recoder(object):
-    """
-    Iterator that reads an encoded stream and reencodes the input to UTF-8
-    """
-    def __init__(self, stream, encoding):
-        self.reader = codecs.getreader(encoding)(stream)
-
-    def __iter__(self):
-        return self
-
-    def __next__(self):
-        "Encode the next line of the file."
-        return self.reader.next().encode("utf-8")
-
-    next = __next__
-
-class UnicodeReader(object):
-    """
-    A CSV reader which will iterate over lines in the CSV file,
-    which is encoded in the given encoding.
-    """
-
-    def __init__(self, csvfile, encoding="utf-8", **kwds):
-        self.first_row = True
-        csvfile = UTF8Recoder(csvfile, encoding)
-        self.reader = csv.reader(csvfile, **kwds)
-
-    def __next__(self):
-        "Read the next line of the file."
-        row = next(self.reader)
-        rowlist = [conv_to_unicode(s, "utf-8") for s in row]
-        # Add check for Byte Order Mark (Windows, Notepad probably):
-        if self.first_row:
-            if len(rowlist) > 0 and rowlist[0].startswith("\ufeff"):
-                rowlist[0] = rowlist[0][1:]
-            self.first_row = False
-        return rowlist
-
-    def __iter__(self):
-        return self
-
-    next = __next__
-
 #-------------------------------------------------------------------------
 #
 # Support and main functions
@@ -133,7 +89,7 @@ def rd(line_number, row, col, key, default = None):
     """ Return Row data by column name """
     if key in col:
         if col[key] >= len(row):
-            LOG.warn("missing '%s, on line %d" % (key, line_number))
+            LOG.warning("missing '%s, on line %d" % (key, line_number))
             return default
         retval = row[col[key]].strip()
         if retval == "":
@@ -148,22 +104,31 @@ def importData(dbase, filename, user):
     if dbase.get_feature("skip-import-additions"): # don't add source or tags
         parser = CSVParser(dbase, user, None)
     else:
-        parser = CSVParser(dbase, user, (config.get('preferences.tag-on-import-format') if 
+        parser = CSVParser(dbase, user, (config.get('preferences.tag-on-import-format') if
                                          config.get('preferences.tag-on-import') else None))
     try:
-        with OpenFileOrStdin(filename, 'b') as filehandle:
+        with open(filename, 'rb') as filehandle:
+            line = filehandle.read(3)
+            if line == codecs.BOM_UTF8:
+                filehandle.seek(0)
+                filehandle = TextIOWrapper(filehandle, encoding='utf_8_sig',
+                                           errors='replace', newline='')
+            else:                       # just open with OS encoding
+                filehandle.seek(0)
+                filehandle = TextIOWrapper(filehandle,
+                                           errors='replace', newline='')
             parser.parse(filehandle)
     except EnvironmentError as err:
         user.notify_error(_("%s could not be opened\n") % filename, str(err))
         return
-    return None # This module doesn't provide info about what got imported.
+    return ImportInfo({_("Results"): _("done")})
 
 #-------------------------------------------------------------------------
 #
-# CSV Parser 
+# CSV Parser
 #
 #-------------------------------------------------------------------------
-class CSVParser(object):
+class CSVParser:
     """Class to read data in CSV format from a file object."""
     def __init__(self, dbase, user, default_tag_format=None):
         self.db = dbase
@@ -173,75 +138,99 @@ class CSVParser(object):
         self.index = 0
         self.fam_count = 0
         self.indi_count = 0
+        self.place_count = 0
         self.pref  = {} # person ref, internal to this sheet
-        self.fref  = {} # family ref, internal to this sheet        
+        self.fref  = {} # family ref, internal to this sheet
+        self.placeref = {}
+        self.place_types = {}
+        # Build reverse dictionary, name to type number
+        for items in PlaceType().get_map().items(): # (0, 'Custom')
+            self.place_types[items[1]] = items[0]
+            self.place_types[items[1].lower()] = items[0]
+            if _(items[1]) != items[1]:
+                self.place_types[_(items[1])] = items[0]
+        # Add custom types:
+        for custom_type in self.db.get_place_types():
+            self.place_types[custom_type] = 0
+            self.place_types[custom_type.lower()] = 0
         column2label = {
-            "surname": ("Lastname", "Surname", _("Surname"), "lastname",
-                "last_name", "surname", _("surname")),
-            "firstname": ("Firstname", "Given name", _("Given name"), "Given",
-                _("Given"), "firstname", "first_name", "given_name",
-                "given name", _("given name"), "given", _("given")),
-            "callname": ("Callname", "Call name", _("Call name"), "Call",
-                _("Call"), "callname", "call_name", "call name", "call",
-                _("call")),
-            "title": ("Title", _("Person|Title"), "title", _("Person|title")),
-            "prefix": ("Prefix", _("Prefix"), "prefix", _("prefix")),
-            "suffix": ("Suffix", _("Suffix"), "suffix", _("suffix")),
-            "gender": ("Gender", _("Gender"), "gender", _("gender")),
-            "source": ("Source", _("Source"), "source", _("source")),
-            "note": ("Note", _("Note"), "note", _("note")),
-            "birthplace": ("Birthplace", "Birth place", _("Birth place"),
-                "birthplace", "birth_place", "birth place", _("birth place")),
-            "birthdate": ("Birthdate", "Birth date", _("Birth date"),
-                "birthdate", "birth_date", "birth date", _("birth date")),
-            "birthsource": ("Birthsource", "Birth source", _("Birth source"),
-                "birthsource", "birth_source", "birth source",
-                _("birth source")),
-            "baptismplace": ("Baptismplace", "Baptism place",
-                _("Baptism place"), "baptismplace", "baptism place",
-                _("baptism place")),
-            "baptismdate": ("Baptismdate", "Baptism date", _("Baptism date"),
-                "baptismdate", "baptism date", _("baptism date")),
-            "baptismsource": ("Baptismsource", "Baptism source",
-                _("Baptism source"), "baptismsource", "baptism source",
-                _("baptism source")),
-            "burialplace": ("Burialplace", "Burial place", _("Burial place"),
-                "burialplace", "burial place", _("burial place")),
-            "burialdate": ("Burialdate", "Burial date", _("Burial date"),
-                "burialdate", "burial date", _("burial date")),
-            "burialsource": ("Burialsource", "Burial source",
-                _("Burial source"), "burialsource", "burial source",
-                _("burial source")),
-            "deathplace": ("Deathplace", "Death place", _("Death place"),
-                "deathplace", "death_place", "death place", _("death place")),
-            "deathdate": ("Deathdate", "Death date", _("Death date"),
-                "deathdate", "death_date", "death date", _("death date")),
-            "deathsource": ("Deathsource", "Death source", _("Death source"),
-                "deathsource", "death_source", "death source",
-                _("death source")),
-            "deathcause": ("Deathcause", "Death cause", _("Death cause"),
-                "deathcause", "death_cause", "death cause", _("death cause")),
-            "grampsid": ("Grampsid", "ID", "Gramps id", _("Gramps ID"),
-                "grampsid", "id", "gramps_id", "gramps id", _("Gramps id")),
-            "person": ("Person", _("Person"), "person", _("person")),
+            "surname": ("lastname", "last_name", "surname", _("surname"),
+                        _("Surname")),
+            "firstname": ("firstname", "first_name", "given_name", "given",
+                          "given name", _("given name"), _("given"),
+                          _("Given"), _("Given name")),
+            "callname": ("call name", _("Call name"), "callname", "call_name",
+                         "call", _("Call"), _("call")),
+            "title": ("title", _("title"), _("Person or Place|title")),
+            "prefix": ("prefix", _("prefix"), _("Prefix")),
+            "suffix": ("suffix", _("suffix"), _("Suffix")),
+            "gender": ("gender", _("gender"), _("Gender")),
+            "source": ("source", _("source"), _("Source")),
+            "note": ("note", _("note"), _("Note")),
+            "birthplace": ("birthplace", "birth_place", "birth place",
+                           _("birth place"), _("Birth place")),
+            "birthplace_id": ("birthplaceid", "birth_place_id",
+                              "birth place id", _("birth place id"),
+                              "birthplace_id"),
+            "birthdate": ("birthdate", "birth_date", "birth date",
+                          _("birth date")),
+            "birthsource": ("birthsource", "birth_source", "birth source",
+                            _("birth source")),
+            "baptismplace": ("baptismplace", "baptism place",
+                             _("baptism place")),
+            "baptismplace_id": ("baptismplaceid", "baptism place id",
+                                _("baptism place id"), "baptism_place_id",
+                                "baptismplace_id"),
+            "baptismdate": ("baptismdate", "baptism date", _("baptism date")),
+            "baptismsource": ("baptismsource", "baptism source",
+                              _("baptism source")),
+            "burialplace": ("burialplace", "burial place", _("burial place")),
+            "burialplace_id": ("burialplaceid", "burial place id",
+                               _("burial place id"), "burial_place_id",
+                               "burialplace_id"),
+            "burialdate": ("burialdate", "burial date", _("burial date")),
+            "burialsource": ("burialsource", "burial source",
+                             _("burial source")),
+            "deathplace": ("deathplace", "death_place", "death place",
+                           _("death place")),
+            "deathplace_id": ("deathplaceid", "death place id",
+                              _("death place id"), "death_place_id",
+                              "deathplace_id"),
+            "deathdate": ("deathdate", "death_date", "death date",
+                          _("death date")),
+            "deathsource": ("deathsource", "death_source", "death source",
+                            _("death source")),
+            "deathcause": ("deathcause", "death_cause", "death cause",
+                           _("death cause")),
+            "grampsid": (_("Gramps ID"), "grampsid", "id", "gramps_id",
+                         "gramps id"),
+            "person": ("person", _("person"), _("Person")),
             # ----------------------------------
-            "child": ("Child", _("Child"), "child", _("child")),
-            "family": ("Family", _("Family"), "family", _("family")),
+            "child": ("child", _("child"), _("Child")),
+            "family": ("family", _("family"), _("Family")),
             # ----------------------------------
-            "wife": ("Mother", _("Mother"), "Wife", _("Wife"), "Parent2",
-                _("Parent2"), "mother", _("mother"), "wife", _("wife"),
-                "parent2", _("parent2")),
-            "husband": ("Father", _("Father"), "Husband", _("Husband"),
-                "Parent1", _("Parent1"), "father", _("father"), "husband",
-                _("husband"), "parent1", _("parent1")),
-            "marriage": ("Marriage", _("Marriage"), "marriage", _("marriage")),
-            "date": ("Date", _("Date"), "date", _("date")),
-            "place": ("Place", _("Place"), "place", _("place")),
-            }
+            "wife": ("mother", _("mother"), _("Mother"),
+                     "wife", _("wife"), _("Wife"),
+                     "parent2", _("parent2")),
+            "husband": ("father", _("father"), _("Father"),
+                        "husband", _("husband"), _("Husband"),
+                        "parent1", _("parent1")),
+            "marriage": ("marriage", _("marriage"), _("Marriage")),
+            "date": ("date", _("date"), _("Date")),
+            "place": ("place", _("place"), _("Place")),
+            "place_id": ("place id", "place_id", "placeid", _("place id")),
+            "name": ("name", _("name"), _("Name")),
+            "type": ("type", _("type"), _("Type")),
+            "latitude": ("latitude", _("latitude")),
+            "longitude": ("longitude", _("longitude")),
+            "code": ("code", _("code"), _("Code")),
+            "enclosed_by": ("enclosed by", _("enclosed by"),
+                            "enclosed_by", _("enclosed_by"), "enclosedby")
+        }
         lab2col_dict = []
         for key in list(column2label.keys()):
             for val in column2label[key]:
-                lab2col_dict.append((val, key))
+                lab2col_dict.append((val.lower(), key))
         self.label2column = dict(lab2col_dict)
         if default_tag_format:
             name = time.strftime(default_tag_format)
@@ -260,12 +249,11 @@ class CSVParser(object):
 
     def read_csv(self, filehandle):
         "Read the data from the file and return it as a list."
-        reader = UnicodeReader(filehandle)
         try:
-            data = [[r.strip() for r in row] for row in reader]
+            data = [[r.strip() for r in row] for row in csv.reader(filehandle)]
         except csv.Error as err:
             self.user.notify_error(_('format error: line %(line)d: %(zero)s') % {
-                        'line' : reader.reader.line_num, 'zero' : err } )
+                        'line' : reader.line_num, 'zero' : err } )
             return None
         return data
 
@@ -284,10 +272,12 @@ class CSVParser(object):
                     return self.lookup(type_, id_)
                 else:
                     return db_lookup
-            elif id_.lower() in self.fref:
-                return self.fref[id_.lower()]
             else:
-                return None
+                id_ = self.db.fid2user_format(id_)
+                if id_.lower() in self.fref:
+                    return self.fref[id_.lower()]
+                else:
+                    return None
         elif type_ == "person":
             if id_.startswith("[") and id_.endswith("]"):
                 id_ = self.db.id2user_format(id_[1:-1])
@@ -296,12 +286,28 @@ class CSVParser(object):
                     return self.lookup(type_, id_)
                 else:
                     return db_lookup
-            elif id_.lower() in self.pref:
-                return self.pref[id_.lower()]
             else:
-                return None
+                id_ = self.db.id2user_format(id_)
+                if id_.lower() in self.pref:
+                    return self.pref[id_.lower()]
+                else:
+                    return None
+        elif type_ == "place":
+            if id_.startswith("[") and id_.endswith("]"):
+                id_ = self.db.pid2user_format(id_[1:-1])
+                db_lookup = self.db.get_place_from_gramps_id(id_)
+                if db_lookup is None:
+                    return self.lookup(type_, id_)
+                else:
+                    return db_lookup
+            else:
+                id_ = self.db.pid2user_format(id_)
+                if id_.lower() in self.placeref:
+                    return self.placeref[id_.lower()]
+                else:
+                    return None
         else:
-            LOG.warn("invalid lookup type in CSV import: '%s'" % type_)
+            LOG.warning("invalid lookup type in CSV import: '%s'" % type_)
             return None
 
     def storeup(self, type_, id_, object_):
@@ -315,8 +321,11 @@ class CSVParser(object):
         elif type_ == "family":
             id_ = self.db.fid2user_format(id_)
             self.fref[id_.lower()] = object_
+        elif type_ == "place":
+            id_ = self.db.pid2user_format(id_)
+            self.placeref[id_.lower()] = object_
         else:
-            LOG.warn("invalid storeup type in CSV import: '%s'" % type_)
+            LOG.warning("invalid storeup type in CSV import: '%s'" % type_)
 
     def parse(self, filehandle):
         """
@@ -325,11 +334,11 @@ class CSVParser(object):
         :param filehandle: open file handle positioned at start of the file
         """
         progress_title = _('CSV Import')
-        with self.user.progress(progress_title, 
+        with self.user.progress(progress_title,
                 _('Reading data...'), 1) as step:
             data = self.read_csv(filehandle)
 
-        with self.user.progress(progress_title, 
+        with self.user.progress(progress_title,
                 _('Importing data...'), len(data)) as step:
             tym = time.time()
             self.db.disable_signals()
@@ -354,8 +363,10 @@ class CSVParser(object):
         self.index = 0
         self.fam_count = 0
         self.indi_count = 0
+        self.place_count = 0
         self.pref  = {} # person ref, internal to this sheet
-        self.fref  = {} # family ref, internal to this sheet        
+        self.fref  = {} # family ref, internal to this sheet
+        self.placeref = {}
         header = None
         line_number = 0
         for row in data:
@@ -366,14 +377,14 @@ class CSVParser(object):
                 continue
             ######################################
             if header is None:
-                header = [self.cleanup_column_name(r) for r in row]
+                header = [self.cleanup_column_name(r.lower()) for r in row]
                 col = {}
                 count = 0
                 for key in header:
                     col[key] = count
                     count += 1
                 continue
-            # three different kinds of data: person, family, and marriage
+            # four different kinds of data: person, family, and marriage
             if (("marriage" in header) or
                 ("husband" in header) or
                 ("wife" in header)):
@@ -382,8 +393,10 @@ class CSVParser(object):
                 self._parse_family(line_number, row, col)
             elif "surname" in header:
                 self._parse_person(line_number, row, col)
+            elif "place" in header:
+                self._parse_place(line_number, row, col)
             else:
-                LOG.warn("ignoring line %d" % line_number)
+                LOG.warning("ignoring line %d" % line_number)
         return None
 
     def _parse_marriage(self, line_number, row, col):
@@ -393,14 +406,15 @@ class CSVParser(object):
         wife     = rd(line_number, row, col, "wife")
         marriagedate = rd(line_number, row, col, "date")
         marriageplace = rd(line_number, row, col, "place")
+        marriageplace_id = rd(line_number, row, col, "place_id")
         marriagesource = rd(line_number, row, col, "source")
         note = rd(line_number, row, col, "note")
         wife = self.lookup("person", wife)
         husband = self.lookup("person", husband)
         if husband is None and wife is None:
             # might have children, so go ahead and add
-            LOG.warn("no parents on line %d; adding family anyway" %
-                     line_number)
+            LOG.warning("no parents on line %d; adding family anyway" %
+                        line_number)
         family = self.get_or_create_family(marriage_ref, husband, wife)
         # adjust gender, if not already provided
         if husband:
@@ -414,13 +428,18 @@ class CSVParser(object):
                 wife.set_gender(Person.FEMALE)
                 self.db.commit_person(wife, self.trans)
         if marriage_ref:
-            self.storeup("family", marriage_ref.lower(), family)
+            self.storeup("family", marriage_ref, family)
         if marriagesource:
             # add, if new
             new, marriagesource = self.get_or_create_source(marriagesource)
+        if marriageplace and marriageplace_id:
+            raise Error("Error in marriage: can't have a place and place_id")
         if marriageplace:
             # add, if new
             new, marriageplace = self.get_or_create_place(marriageplace)
+        elif marriageplace_id:
+            # better exist already, locally or in database:
+            marriageplace = self.lookup("place", marriageplace_id)
         if marriagedate:
             marriagedate = _dp.parse(marriagedate)
         if marriagedate or marriageplace or marriagesource or note:
@@ -465,8 +484,8 @@ class CSVParser(object):
         "Parse the content of a family line"
         family_ref   = rd(line_number, row, col, "family")
         if family_ref is None:
-            LOG.warn("no family reference found for family on line %d" %
-                     line_number)
+            LOG.warning("no family reference found for family on line %d" %
+                        line_number)
             return # required
         child   = rd(line_number, row, col, "child")
         source  = rd(line_number, row, col, "source")
@@ -475,12 +494,12 @@ class CSVParser(object):
         child = self.lookup("person", child)
         family = self.lookup("family", family_ref)
         if family is None:
-            LOG.warn("no matching family reference found for family "
-                     "on line %d" % line_number)
+            LOG.warning("no matching family reference found for family "
+                        "on line %d" % line_number)
             return
         if child is None:
-            LOG.warn("no matching child reference found for family "
-                     "on line %d" % line_number)
+            LOG.warning("no matching child reference found for family "
+                        "on line %d" % line_number)
             return
         # is this child already in this family? If so, don't add
         LOG.debug("children: %s", [ref.ref for ref in
@@ -549,15 +568,19 @@ class CSVParser(object):
         source    = rd(line_number, row, col, "source")
         note      = rd(line_number, row, col, "note")
         birthplace  = rd(line_number, row, col, "birthplace")
+        birthplace_id  = rd(line_number, row, col, "birthplace_id")
         birthdate   = rd(line_number, row, col, "birthdate")
         birthsource = rd(line_number, row, col, "birthsource")
         baptismplace  = rd(line_number, row, col, "baptismplace")
+        baptismplace_id  = rd(line_number, row, col, "baptismplace_id")
         baptismdate   = rd(line_number, row, col, "baptismdate")
         baptismsource = rd(line_number, row, col, "baptismsource")
         burialplace  = rd(line_number, row, col, "burialplace")
+        burialplace_id  = rd(line_number, row, col, "burialplace_id")
         burialdate   = rd(line_number, row, col, "burialdate")
         burialsource = rd(line_number, row, col, "burialsource")
         deathplace  = rd(line_number, row, col, "deathplace")
+        deathplace_id  = rd(line_number, row, col, "deathplace_id")
         deathdate   = rd(line_number, row, col, "deathdate")
         deathsource = rd(line_number, row, col, "deathsource")
         deathcause  = rd(line_number, row, col, "deathcause")
@@ -568,8 +591,8 @@ class CSVParser(object):
         person = self.lookup("person", person_ref)
         if person is None:
             if surname is None:
-                LOG.warn("empty surname for new person on line %d" %
-                         line_number)
+                LOG.warning("empty surname for new person on line %d" %
+                            line_number)
                 surname = ""
             # new person
             person = self.create_person()
@@ -643,13 +666,18 @@ class CSVParser(object):
         # Birth:
         if birthdate is not None:
             birthdate = _dp.parse(birthdate)
+        if birthplace and birthplace_id:
+            raise Error("Error in person: can't have a birthplace and birthplace_id")
         if birthplace is not None:
             new, birthplace = self.get_or_create_place(birthplace)
+        elif birthplace_id:
+            # better exist already, locally or in database:
+            birthplace = self.lookup("place", birthplace_id)
         if birthsource is not None:
             new, birthsource = self.get_or_create_source(birthsource)
         if birthdate or birthplace or birthsource:
-            new, birth = self.get_or_create_event(person, 
-                 EventType.BIRTH, birthdate, 
+            new, birth = self.get_or_create_event(person,
+                 EventType.BIRTH, birthdate,
                  birthplace, birthsource)
             birth_ref = person.get_birth_ref()
             if birth_ref is None:
@@ -660,13 +688,18 @@ class CSVParser(object):
         # Baptism:
         if baptismdate is not None:
             baptismdate = _dp.parse(baptismdate)
+        if baptismplace and baptismplace_id:
+            raise Error("Error in person: can't have a baptismplace and baptismplace_id")
         if baptismplace is not None:
             new, baptismplace = self.get_or_create_place(baptismplace)
+        elif baptismplace_id:
+            # better exist already, locally or in database:
+            baptismplace = self.lookup("place", baptismplace_id)
         if baptismsource is not None:
             new, baptismsource = self.get_or_create_source(baptismsource)
         if baptismdate or baptismplace or baptismsource:
-            new, baptism = self.get_or_create_event(person, 
-                 EventType.BAPTISM, baptismdate, 
+            new, baptism = self.get_or_create_event(person,
+                 EventType.BAPTISM, baptismdate,
                  baptismplace, baptismsource)
             baptism_ref = get_primary_event_ref_from_type(self.db, person,
                                                           "Baptism")
@@ -678,8 +711,13 @@ class CSVParser(object):
         # Death:
         if deathdate is not None:
             deathdate = _dp.parse(deathdate)
+        if deathplace and deathplace_id:
+            raise Error("Error in person: can't have a deathplace and deathplace_id")
         if deathplace is not None:
             new, deathplace = self.get_or_create_place(deathplace)
+        elif deathplace_id:
+            # better exist already, locally or in database:
+            deathplace = self.lookup("place", deathplace_id)
         if deathsource is not None:
             new, deathsource = self.get_or_create_source(deathsource)
         if deathdate or deathplace or deathsource or deathcause:
@@ -698,13 +736,18 @@ class CSVParser(object):
         # Burial:
         if burialdate is not None:
             burialdate = _dp.parse(burialdate)
+        if burialplace and burialplace_id:
+            raise Error("Error in person: can't have a burialplace and burialplace_id")
         if burialplace is not None:
             new, burialplace = self.get_or_create_place(burialplace)
+        elif burialplace_id:
+            # better exist already, locally or in database:
+            burialplace = self.lookup("place", burialplace_id)
         if burialsource is not None:
             new, burialsource = self.get_or_create_source(burialsource)
         if burialdate or burialplace or burialsource:
-            new, burial = self.get_or_create_event(person, 
-                 EventType.BURIAL, burialdate, 
+            new, burial = self.get_or_create_event(person,
+                 EventType.BURIAL, burialdate,
                  burialplace, burialsource)
             burial_ref = get_primary_event_ref_from_type(self.db, person,
                                                          "Burial")
@@ -719,6 +762,59 @@ class CSVParser(object):
             self.find_and_set_citation(person, source)
         self.db.commit_person(person, self.trans)
 
+    def _parse_place(self, line_number, row, col):
+        "Parse the content of a Place line."
+        place_id = rd(line_number, row, col, "place")
+        place_title = rd(line_number, row, col, "title")
+        place_name = rd(line_number, row, col, "name")
+        place_type_str = rd(line_number, row, col, "type")
+        place_latitude = rd(line_number, row, col, "latitude")
+        place_longitude = rd(line_number, row, col, "longitude")
+        place_code = rd(line_number, row, col, "code")
+        place_enclosed_by_id = rd(line_number, row, col, "enclosed_by")
+        place_date = rd(line_number, row, col, "date")
+        #########################################################
+        # if this place already exists, don't create it
+        place = self.lookup("place", place_id)
+        if place is None:
+            # new place
+            place = self.create_place()
+            if place_id is not None:
+                if place_id.startswith("[") and place_id.endswith("]"):
+                    place.gramps_id = self.db.pid2user_format(place_id[1:-1])
+                self.storeup("place", place_id, place)
+        if place_title is not None:
+            place.title = place_title
+        if place_name is not None:
+            place.name = PlaceName(value=place_name)
+        if place_type_str is not None:
+            place.place_type = self.get_place_type(place_type_str)
+        if place_latitude is not None:
+            place.lat = place_latitude
+        if place_longitude is not None:
+            place.long = place_longitude
+        if place_code is not None:
+            place.code = place_code
+        if place_enclosed_by_id is not None:
+            place_enclosed_by = self.lookup("place", place_enclosed_by_id)
+            if place_enclosed_by is None:
+                raise Exception("cannot enclose %s in %s as it doesn't exist" % (place.gramps_id, place_enclosed_by_id))
+            if not place_enclosed_by.handle in place.placeref_list:
+                placeref = PlaceRef()
+                placeref.ref = place_enclosed_by.handle
+                if place_date:
+                    placeref.date = _dp.parse(place_date)
+                place.placeref_list.append(placeref)
+        #########################################################
+        self.db.commit_place(place, self.trans)
+
+    def get_place_type(self, place_type_str):
+        if place_type_str in self.place_types:
+            return PlaceType((self.place_types[place_type_str], place_type_str))
+        else:
+            # New custom type:
+            return PlaceType((0, place_type_str))
+
     def get_or_create_family(self, family_ref, husband, wife):
         "Return the family object for the give family ID."
         # if a gramps_id and exists:
@@ -765,7 +861,7 @@ class CSVParser(object):
             self.db.commit_person(wife, self.trans)
         self.fam_count += 1
         return family
-        
+
     def get_or_create_event(self, object_, type_, date=None, place=None,
                             source=None):
         """ Add or find a type event on object """
@@ -802,7 +898,7 @@ class CSVParser(object):
             self.find_and_set_citation(event, source)
         self.db.add_event(event, self.trans)
         return (1, event)
-    
+
     def create_person(self):
         """ Used to create a new person we know doesn't exist """
         person = Person()
@@ -812,15 +908,26 @@ class CSVParser(object):
         self.indi_count += 1
         return person
 
+    def create_place(self):
+        """ Used to create a new person we know doesn't exist """
+        place = Place()
+        if self.default_tag:
+            place.add_tag(self.default_tag.handle)
+        self.db.add_place(place, self.trans)
+        self.place_count += 1
+        return place
+
     def get_or_create_place(self, place_name):
         "Return the requested place object tuple-packed with a new indicator."
         LOG.debug("get_or_create_place: looking for: %s", place_name)
         for place_handle in self.db.iter_place_handles():
             place = self.db.get_place_from_handle(place_handle)
-            if place.get_title() == place_name:
+            place_title = place_displayer.display(self.db, place)
+            if place_title == place_name:
                 return (0, place)
         place = Place()
         place.set_title(place_name)
+        place.name = PlaceName(value=place_name)
         self.db.add_place(place, self.trans)
         return (1, place)
 
@@ -861,6 +968,6 @@ class CSVParser(object):
         LOG.debug("   creating citation")
         citation.set_reference_handle(source.get_handle())
         self.db.add_citation(citation, self.trans)
-        LOG.debug("   created citation, citation %s %s" % 
+        LOG.debug("   created citation, citation %s %s" %
                   (citation, citation.get_gramps_id()))
         obj.add_citation(citation.get_handle())
