@@ -18,7 +18,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
 "Find unused objects and remove with the user's permission."
@@ -28,10 +28,10 @@
 # python modules
 #
 #-------------------------------------------------------------------------
-from __future__ import with_statement
 from gramps.gen.const import GRAMPS_LOCALE as glocale
 _ = glocale.translation.gettext
 from gramps.gen.constfunc import handle2internal
+from gramps.gen.filters import GenericFilterFactory, rules
 
 #------------------------------------------------------------------------
 #
@@ -59,6 +59,7 @@ from gramps.gen.db import DbTxn
 from gramps.gen.errors import WindowActiveError
 from gramps.gui.managedwindow import ManagedWindow
 from gramps.gen.datehandler import displayer as _dd
+from gramps.gen.display.place import displayer as _pd
 from gramps.gen.updatecallback import UpdateCallback
 from gramps.gui.plug import tool
 from gramps.gui.glade import Glade
@@ -95,37 +96,37 @@ class RemoveUnused(tool.Tool, ManagedWindow, UpdateCallback):
                          'remove'  : self.db.remove_event,
                          'get_text': self.get_event_text,
                          'editor'  : 'EditEvent',
-                         'stock'   : 'gramps-event',
+                         'icon'    : 'gramps-event',
                          'name_ix' : 4},
             'sources' : {'get_func': self.db.get_source_from_handle,
                          'remove'  : self.db.remove_source,
                          'get_text': None,
                          'editor'  : 'EditSource',
-                         'stock'   : 'gramps-source',
+                         'icon'    : 'gramps-source',
                          'name_ix' : 2},
             'places'  : {'get_func': self.db.get_place_from_handle,
                          'remove'  : self.db.remove_place,
                          'get_text': None,
                          'editor'  : 'EditPlace',
-                         'stock'   : 'gramps-place',
+                         'icon'    : 'gramps-place',
                          'name_ix' : 2},
             'media'   : {'get_func': self.db.get_object_from_handle,
                          'remove'  : self.db.remove_object,
                          'get_text': None,
                          'editor'  : 'EditMedia',
-                         'stock'   : 'gramps-media',
+                         'icon'    : 'gramps-media',
                          'name_ix' : 4},
             'repos'   : {'get_func': self.db.get_repository_from_handle,
                          'remove'  : self.db.remove_repository,
                          'get_text': None,
                          'editor'  : 'EditRepository',
-                         'stock'   : 'gramps-repository',
+                         'icon'    : 'gramps-repository',
                          'name_ix' : 3},
             'notes'   : {'get_func': self.db.get_note_from_handle,
                          'remove'  : self.db.remove_note,
                          'get_text': self.get_note_text,
                          'editor'  : 'EditNote',
-                         'stock'   : 'gramps-notes',
+                         'icon'    : 'gramps-notes',
                          'name_ix' : 2},
             }
 
@@ -267,6 +268,15 @@ class RemoveUnused(tool.Tool, ManagedWindow, UpdateCallback):
             ('notes', db.get_note_cursor, db.get_number_of_notes),
             )
 
+        # bug 7619 : don't select notes from to do list.
+        # notes associated to the todo list doesn't have references.
+        # get the todo list (from get_note_list method of the todo gramplet )
+        all_notes = self.dbstate.db.get_note_handles()
+        FilterClass = GenericFilterFactory('Note')
+        filter = FilterClass()
+        filter.add_rule(rules.note.HasType(["To Do"]))
+        todo_list = filter.apply(self.dbstate.db, all_notes)
+
         for (the_type, cursor_func, total_func) in tables:
             if not self.options.handler.options_dict[the_type]:
                 # This table was not requested. Skip it.
@@ -277,8 +287,9 @@ class RemoveUnused(tool.Tool, ManagedWindow, UpdateCallback):
                 fbh = db.find_backlink_handles
                 for handle, data in cursor:
                     if not any(h for h in fbh(handle)):
-                        self.add_results((the_type, handle2internal(handle), 
-                                          data))
+                        if not (handle in todo_list and the_type == "notes"):
+                            self.add_results((the_type, handle2internal(handle), 
+                                              data))
                     self.update()
             self.reset()
 
@@ -352,8 +363,8 @@ class RemoveUnused(tool.Tool, ManagedWindow, UpdateCallback):
 
     def get_image(self, column, cell, model, iter, user_data=None):
         the_type = model.get_value(iter, RemoveUnused.OBJ_TYPE_COL)
-        the_stock = self.tables[the_type]['stock']
-        cell.set_property('stock-id', the_stock)
+        the_icon = self.tables[the_type]['icon']
+        cell.set_property('icon-name', the_icon)
 
     def add_results(self, results):
         (the_type, handle, data) = results
@@ -398,10 +409,8 @@ class RemoveUnused(tool.Tool, ManagedWindow, UpdateCallback):
                 text += '; %s' % date
 
             # see if there is a place
-            place_handle = event.get_place_handle()
-            if place_handle:
-                place = self.db.get_place_from_handle(place_handle)
-                text += '; %s' % place.get_title()
+            if event.get_place_handle():
+                text += '; %s' % _pd.display_event(self.db, event)
 
         return text
         
